##
#	This script computes the formants of all labeled
#	segments of all wav-soundfiles in a directory.
##

clearinfo

#
## 1) Inquire some parameters
## ! Note that 'form' may only be used once in a script!
#

form Formant parameters:
		sentence directory C:\Praat\analiz\
		word result_file Formants_all.txt
	      comment ______________________________________________________________________________________________________________
		optionmenu tier: 1
		option Tier 1
		option Tier 2
		option Tier 3
		option Tier 4
		option Tier 5
		word label
		real min_length 25
	      comment ______________________________________________________________________________________________________________
		boolean Center 1
		boolean Edges 0
		boolean Mean 0
	      comment ______________________________________________________________________________________________________________
		integer max_freq 5000
		integer nr_formants 5
		real window_length 0.025
	      comment ______________________________________________________________________________________________________________
		choice Unit: 1
		button Hertz
		button Bark
endform

# LPC parameters that are rarely changed (so they are set here, to give space in the form window)
step_rate = 0.005
pre_emphasis = 50

# move window by half width at edges of intervals (might be required later)
window_half_length = window_length / 2.0

# check whether anything is seletced at all
if (center = 0) and (edges = 0) and (mean = 0) 
	printline Neither Center, Edges nor Mean selected: no data computed.
	exit
endif

# recode units (and strings for header of output file)
if (unit = 1)
	unit$ = "Hertz"
	un$ = ""
else
	unit$ = "Bark"
	un$ = "[Bk]"
endif

#
## 2) delete any pre-existing result file and write a header line
#

result_file$ = directory$+result_file$
filedelete 'result_file$'

fileappend 'result_file$' File'tab$'Label'tab$'Beg[s]'tab$'Dur[ms]
	
if center = 1
	for i_formant from 1 to nr_formants
		fileappend 'result_file$' 'tab$'F'i_formant:0'_mid'un$''tab$'B'i_formant:0'_mid'un$'
	endfor
endif
if edges = 1
	for i_formant from 1 to nr_formants
		fileappend 'result_file$' 'tab$'F'i_formant:0'_left'un$''tab$'B'i_formant:0'_left'un$''tab$'F'i_formant:0'_right'un$''tab$'B'i_formant:0'_right'un$'
	endfor
endif
if mean = 1
	for i_formant from 1 to nr_formants
		fileappend 'result_file$' 'tab$'F'i_formant:0'_mean'un$''tab$'F'i_formant:0'_sd'un$'
	endfor
endif
fileappend 'result_file$' 'newline$'

#
## 3) Get file names from a directory
##	In case the whole file should be measured, look for all wav files. 
##	If durations of labels are required, search for TextGrid files only. 
#

# nr. of files that failed
nr_no_grid = 0					
nr_no_wav = 0					

# total nr. of segments of all files
tot_segments = 0

# create list of .textgrid files
Create Strings as file list...  file_list 'directory$'*.TextGrid
nr_files = Get number of strings

#
## 4) Go thru all files
#

for i_file to nr_files
	select Strings file_list
	file_name$ = Get string... i_file
	Read from file... 'directory$''file_name$'
	
# get length of all segments of one textgrid
	base_name$ = selected$("TextGrid")
	print Handling 'base_name$' 

# check whether the selected tier number is not too large and whether selected tier is an interval tier.
	max_tier = Get number of tiers
	tier_is_interval = 0
	if max_tier >= tier
		tier_is_interval = Is interval tier... tier
	endif
		
# next statement is only true if there is an interval tier under investigation
	if  tier_is_interval = 1
		ext1$ = ".wav"
		sound_file_name$ = directory$+base_name$+ext1$
		if fileReadable (sound_file_name$) 
# if any of the above attempts to open a file worked, this will work as well 
			Read from file... 'sound_file_name$'
			To Formant (burg)... step_rate nr_formants max_freq window_length pre_emphasis
# Use the TextGrid to find all segments.
			select TextGrid 'base_name$'
			nr_segments = Get number of intervals... tier
			nr_measured_segments = 0
				
# go thru all segments
			for i_segment to nr_segments
				select TextGrid 'base_name$'
				interval_label$ = Get label of interval... tier i_segment
					
# report all intervals or only labeled intervals (whatever was selected in the form at the beginning)
				if (((label$="") and (interval_label$ <> "") and (interval_label$ <> ".")) or ((label$<>"") and (interval_label$ = label$))) 
# measure length		
					t_left 	 = Get starting point... tier i_segment
					t_right  = Get end point...      tier i_segment
					t_middle = (t_left+t_right) / 2
					duration = (t_right - t_left) * 1000

# compute and report data only it segment duration is larger than a minimal size
					if (duration > min_length)
						nr_measured_segments += 1
						fileappend 'result_file$' 'base_name$''tab$''interval_label$''tab$''t_left:4''tab$''duration:3'

						select Formant 'base_name$'
#
# compute center formants (with median smoothing)
#
						if center = 1
							t_middle_left = t_middle - step_rate
							t_middle_right = t_middle + step_rate
							for i_formant to nr_formants

# Get three formant values at and around the middle of the segment and pick the median

								f_middle_left   = Get value at time... i_formant t_middle_left  'unit$' Linear
								f_middle	      = Get value at time... i_formant t_middle	      'unit$' Linear
								f_middle_right  = Get value at time... i_formant t_middle_right 'unit$' Linear
								b_middle_left   = Get bandwidth at time... i_formant t_middle_left  'unit$' Linear
								b_middle	      = Get bandwidth at time... i_formant t_middle	      'unit$' Linear
								b_middle_right  = Get bandwidth at time... i_formant t_middle_right 'unit$' Linear
								call MedianSmooth f_middle_left f_middle f_middle_right b_middle_left b_middle b_middle_right

# append the data to the result file with no decimals

								if f_middle <> undefined
									if (unit$ = "Hertz")
										fileappend 'result_file$' 'tab$''f_middle:0''tab$''b_middle:0'
									else
										fileappend 'result_file$' 'tab$''f_middle:2''tab$''b_middle:2'
									endif
								else
									fileappend 'result_file$' 'tab$'.'tab$'.
								endif

							endfor				# going thru all formants
						endif						# center formants wanted
#
# compute formants at the edges (without smoothing)
#
						if edges = 1

# compute adjusted positions for edges
							t_adj_left  = t_left + window_half_length
							t_adj_right = t_right - window_half_length

							for i_formant to nr_formants

# Get three formant values at and around the middle of the segment and pick the median

								f_left  = Get value at time... i_formant t_adj_left  'unit$' Linear
								f_right = Get value at time... i_formant t_adj_right 'unit$' Linear
								b_left  = Get bandwidth at time... i_formant t_adj_left  'unit$' Linear
								b_right = Get bandwidth at time... i_formant t_adj_right 'unit$' Linear
# append the data to the result file with no decimals
								if f_left <> undefined
									if (unit$ = "Hertz")
										fileappend 'result_file$' 'tab$''f_left:0''tab$''b_left:0'
									else
										fileappend 'result_file$' 'tab$''f_left:2''tab$''b_left:2'
									endif
								else
									fileappend 'result_file$' 'tab$'.'tab$'.
								endif
								if f_right <> undefined
									if (unit$ = "Hertz")
										fileappend 'result_file$' 'tab$''f_right:0''tab$''b_right:0'
									else
										fileappend 'result_file$' 'tab$''f_right:2''tab$''b_right:2'
									endif
								else
									fileappend 'result_file$' 'tab$'.'tab$'.
								endif

							endfor				# going thru all formants
						endif						# edges info wanted
#
# compute mean formant values
#
						if mean = 1
							for i_formant to nr_formants

# Get the mean of the formant frequency for this segment
								f_mean  = Get mean...               i_formant t_left t_right 'unit$'
								f_stdev = Get standard deviation... i_formant t_left t_right 'unit$'
								if f_mean <> undefined
									if (unit$ = "Hertz")
										fileappend 'result_file$' 'tab$''f_mean:0'
									else
										fileappend 'result_file$' 'tab$''f_mean:2'
									endif
								else
									fileappend 'result_file$' 'tab$'.
								endif
								if f_stdev <> undefined
									if (unit$ = "Hertz")
										fileappend 'result_file$' 'tab$''f_stdev:2'
									else
										fileappend 'result_file$' 'tab$''f_stdev:2'
									endif
								else
									fileappend 'result_file$' 'tab$'.
								endif

							endfor						# going thru formants
						endif								# mean of formants wanted

# all computations doen for one segment				
					fileappend 'result_file$' 'newline$'
					endif								# skipping segments that are too short
				endif									# handling one segment
			endfor									# going thru all intervals
# entertain user
			tot_segments += nr_measured_segments
			perc = (i_file/nr_files) * 100
			printline with 'nr_measured_segments' segments finished ('perc:2'%).
			select TextGrid 'base_name$'
			plus Formant 'base_name$'
			plus Sound 'base_name$'
			Remove
		else											# wav-file missing
			printline skipped since there is no 'base_name$''ext1$' file.
			nr_no_wav += 1
			select TextGrid 'base_name$'
			Remove
		endif											# check for wav-file
	else												# tier missing
		printline skipped since tier 'tier' is not an interval tier.
		nr_no_grid += 1
		select TextGrid 'base_name$'
		Remove
	endif												# check for tier is an interval tier
endfor												# going thru all files

# clean up

select Strings file_list
Remove

# inform user that we are done.
nr_files = nr_files - nr_no_grid - nr_no_wav
printline 'newline$''nr_files' files with a total of 'tot_segments' segments processed.
if nr_no_grid <> 0
	printline 'nr_no_grid' files not processed since the selected tier 'tier' was not an interval tier.
endif
if nr_no_wav <> 0
	printline 'nr_no_wav' files not processed since there were no wav files.
endif
printline Results are written to 'directory$''result_file$'. 'newline$'Program completed.


# Procedure to perform a median smoothing on three values (i.e. pick the middle value of three)
#	Note this procedure is not fully save - if all three values are 'jumpy' it just picks the middle one,
#	which might be completely wrong; the same is true if there are only two or values and the third is 
#	'undefined'. In that case, the middle one is chosen (even if the two others are close to each other).
#	It seems to make better to include some sort of frequency limit (like 100 Hz or less than 5% of
#	the mean) to depart values from each other, or so.

procedure MedianSmooth left mid right bleft bmid bright
	if (left <> undefined) and (mid <> undefined) and (right <> undefined)
		if mid > left
			if mid > right
				if left < right
					mid = right
					bmid = bright
				else
					mid = left
					bmid = bleft
				endif
			endif
		else
			if left < right
				mid = left
				bmid = bleft
			else
				if mid < right
					mid = right
					bmid = bright
				endif
			endif
		endif
	endif
endproc
