##
#	This script opens all sound files in a directory and their associated TextGrids,
#	computes the mean intensity of all segments that have a name
#	and writes the results together with the durations of these segment to the 
#	text file "intensity_mean_results.txt" at the same directory of the sound file.
#
#	This script is not very efficient, since it computes the intensity for the whole files and looks only
#	then for labeled segments - it would be usually much more efficient to compute
#	the intensity only for these segments.
#
##

clearinfo

## 1) Inquire some parameters
# ! Note that 'form' may only be used once in a script!

form Intensity parameters:
	comment Leave the directory path empty if you want to use the current directory.
	sentence directory C:\Praat\analiz\

	comment Which tier should be analyzed?
	integer tier 1
	comment ______________________________________________________________________________________________________________
	real step_rate 0
	real low_f0 50.0
endform

## 2) delete any pre-existing result file and write a header line

result_file$ = directory$+"intensity_mean_results.txt"
filedelete 'result_file$'
fileappend 'result_file$' File'tab$'Label'tab$'Beg[s]'tab$'Dur[ms]'tab$'RMS [dB]'newline$'

#  3) Get file names from a directory

Create Strings as file list...  file_list 'directory$'*.TextGrid
number_of_files = Get number of strings
nr_completed = 0
for i_file to number_of_files
	select Strings file_list
	grid_file_name$ = Get string... i_file
	Read from file... 'directory$''grid_file_name$'
	base_name$ = selected$("TextGrid")
	print Handling 'base_name$' 

# try to open .wav files for this TextGrid

	ext1$ = ".wav"
	sound_file_name$ = directory$+base_name$+ext1$
	if fileReadable (sound_file_name$) 
		Read from file... 'sound_file_name$'
		
# check whether tier is an interval tier.
		
		select TextGrid 'base_name$'
		tier_one_is_interval = Is interval tier... tier
		if  tier_one_is_interval <> 1
			print 'newline$' Tier 'tier' of 'base_name$'.TextGrid is not an interval tier. File ignored.  <*** 'newline$' 
		else
			
# Compute the intensity of the selected sound. 
			
			select Sound 'base_name$'
			To Intensity... low_f0 step_rate yes
			
# Use the TextGrid to find all labeled segments.
			
			select TextGrid 'base_name$'
			nr_segments = Get number of intervals... tier
			for i to nr_segments
				select TextGrid 'base_name$'
				interval_label$ = Get label of interval... tier i
				if interval_label$ <> ""
				
# Get the mean intensity value 
					
					begin_segment = Get starting point... tier i
					end_segment   = Get end point...      tier i
					duration = (end_segment - begin_segment) * 1000
					fileappend 'result_file$' 'base_name$''tab$''interval_label$''tab$''begin_segment:4''tab$''duration:3'
					select Intensity 'base_name$'
					rms = Get mean... begin_segment end_segment dB
					if rms <> undefined
						fileappend 'result_file$' 'tab$''rms:2'
					else
						fileappend 'result_file$' 'tab$'.
					endif
					fileappend 'result_file$' 'newline$'
				endif
			endfor
			
			printline finished. 
			select Sound 'base_name$'
			plus TextGrid 'base_name$'
			plus Intensity 'base_name$'
			Remove
			nr_completed += 1
		endif

	else
		printline failed! No sound file 'directory$''base_name$''ext1$' or *'ext2$' found. <***
		Remove
	endif
endfor

# clear up 

select Strings file_list
Remove

printline 'nr_completed' of 'number_of_files' TextGrid files processed.
printline Intensity data written to 'result_file$'.
printline Done.
