##
#	This script computes the formants of all labeled
#	segments of all wav-soundfiles in a directory.
#
##

clearinfo

#
## 1) Inquire some parameters
## ! Note that 'form' may only be used once in a script!
#

form Spectrum parameters:
	comment Leave the directory path empty if you want to use the current directory.
		sentence directory C:\Praat\spektral\
		word result_file Spectrum_all.txt
	comment ______________________________________________________________________________________________________________
	comment Which interval tiers should be analyzed?
	optionmenu tier: 1
		option Tier 1
		option Tier 2
		option Tier 3
		option Tier 4
		option Tier 5
	comment Leave the label field empty if you want to analyze all labelled intervals
	comment or type the label you want to analyze (only one is possible at a time).
		word label
endform

#
## 2) delete any pre-existing result file and write a header line
#

result_file$ = directory$+result_file$
filedelete 'result_file$'
fileappend 'result_file$' File'tab$'Label'tab$'Beg[s]'tab$'Dur[ms]'tab$'CoG[Hz]'tab$'SD[Hz]'tab$'Skewn'tab$'Kurtos'newline$'

#
## 3) Get file names from a directory
#	Search for TextGrid files only. 
#

# nr. of files that failed
nr_no_grid = 0					
nr_no_wav = 0					

# total nr. of segments of all files
tot_segments = 0

# create list of .textgrid files
Create Strings as file list...  file_list 'directory$'*.TextGrid
nr_files = Get number of strings

#
## 4) Go thru all files
#

for i_file from 1 to nr_files
	select Strings file_list
	file_name$ = Get string... 'i_file'
	Read from file... 'directory$''file_name$'
	
# get length of all segments of one textgrid
	base_name$ = selected$ ("TextGrid")
	print Handling 'base_name$' 

# check whether the selected tier number is not too large and whether selected tier is an interval tier.
	max_tier = Get number of tiers
	tier_is_interval = 0
	if max_tier >= tier
		tier_is_interval = Is interval tier... 'tier'
	endif
		
# next statement is only true if there is an interval tier under investigation
	if  tier_is_interval = 1
		ext1$ = ".wav"
		sound_file_name$ = directory$+base_name$+ext1$
		if fileReadable (sound_file_name$) 
# if any of the above attempts to open a file worked, this will work as well 
			Read from file... 'sound_file_name$'

# Use the TextGrid to find all segments.
			select TextGrid 'base_name$'
			nr_segments = Get number of intervals... 'tier'
			nr_measured_segments = 0
				
# go thru all segments
			for i_segment from 1 to nr_segments
				select TextGrid 'base_name$'
				interval_label$ = Get label of interval... 'tier' 'i_segment'
					
# report all intervals or only labeled intervals (whatever was selected in the form at the beginning)
				if (((label$="") and (interval_label$ <> "")) or ((label$<>"") and (interval_label$ = label$))) 
# measure length		
					nr_measured_segments += 1
					t_left 	 = Get starting point... 'tier' 'i_segment'
					t_right  = Get end point...      'tier' 'i_segment'
					duration = (t_right - t_left) * 1000
					fileappend 'result_file$' 'base_name$''tab$''interval_label$''tab$''t_left:4''tab$''duration:3'
#
# cut out one interval and compute spectrum from it
#
					select Sound 'base_name$'
					Extract part... t_left t_right Rectangular 1 no
					select Sound 'base_name$'_part
					To Spectrum... yes
					mean = Get centre of gravity... 2
					stdev = Get standard deviation... 2
					skew = Get skewness... 2
					kurt = Get kurtosis... 2

					call PrintValue mean
					call PrintValue stdev
					call PrintValue skew
					call PrintValue kurt
# all computations doen for one segment				
					fileappend 'result_file$' 'newline$'
					select Sound 'base_name$'_part
					plus Spectrum 'base_name$'_part
					Remove
				endif								# handling one segment
			endfor									# going thru all intervals
# entertain user
			tot_segments += nr_measured_segments
			perc = (i_file/nr_files) * 100
			printline with 'nr_measured_segments' segments finished ('perc:2'%).
			select TextGrid 'base_name$'
			plus Sound 'base_name$'
			Remove
		else						# wav-file missing
			printline skipped since there is no 'base_name$''ext1$' file.
			nr_no_wav += 1
			select TextGrid 'base_name$'
			Remove
		endif						# check for wav-file
	else							# tier missing
		printline skipped since tier 'tier' is not an interval tier.
		nr_no_grid += 1
		select TextGrid 'base_name$'
		Remove
	endif							# check for tier is an interval tier
endfor								# going thru all files

# clean up

select Strings file_list
Remove

# inform user that we are done.
nr_files = nr_files - nr_no_grid - nr_no_wav
printline 'newline$''nr_files' files with a total of 'tot_segments' segments processed.
if nr_no_grid <> 0
	printline 'nr_no_grid' files not processed since the selected tier 'tier' was not an interval tier.
endif
if nr_no_wav <> 0
	printline 'nr_no_wav' files not processed since there were no wav files.
endif
printline Results are written to 'directory$''result_file$'. 'newline$'Program completed.


# Procedure to print values

procedure PrintValue value
	if value <> undefined
		fileappend 'result_file$' 'tab$''value:2'
	else
		fileappend 'result_file$' 'tab$'.
	endif
endproc
