# histogram.praat ---
# Praat include file containing procedures for working with histograms 

# Author: Piet Mertens
# 2005-03-09	initial implementation
# 2006-01-14	added histogram_addparam
# 2008-11-03	praat style
# 2010-06-17	modified histogram_draw
# Last modification: 2010-06-17


#clearinfo
#call histogram_test


procedure histogram_test
   # Create a table with 1 column and set values from -4
      Create TableOfReal... dummy_name 10 1	; rows columns
      t = selected ("TableOfReal",-1)
      for i to 10
         Set value... i 1 'i'-5		; row column value
      endfor
   # Create a histogram and fill it with values from table
   call histogram_create 10 -5 5
   h = result
   call histogram_addtable h t 1
   call histogram_draw h 0 0
   call histogram_quantile h 0.25
   printline quantile 0.25 = 'result'
   call histogram_quantile h 0.5
   printline quantile 0.5  = 'result'
   call histogram_quantile h 0.75
   printline quantile 0.75 = 'result'
   call histogram_quantile h 0.1
   printline quantile 0.1  = 'result'
   call histogram_remove h
   select 't'
   Remove
endproc


procedure histogram_create nrofbins xstart xend
# Create a histogram, with <nrofbins> bins in range <xstart>..<xend> 
# Return the ID of the histogram
   name$ = "histogram"
   hidden_data$ = "hidden_data" + name$
   hidden_meta$ = "hidden_meta" + name$
   # Create a table for the bins
   Create TableOfReal... hidden_data$ nrofbins 1
   iddata = selected ("TableOfReal",-1)
   # Create a table for the metadata
   Create TableOfReal... hidden_meta$ 6 1
   idmeta = selected ("TableOfReal",-1)
   nrofcounts = 0
   dx = (xend - xstart) / nrofbins
   #Set value... 'row' 'col' 'value'
   Set value... 1 1 iddata
   Set value... 2 1 nrofbins
   Set value... 3 1 xstart
   Set value... 4 1 xend
   Set value... 5 1 nrofcounts
   Set value... 6 1 dx
   result = idmeta
endproc


procedure histogram_remove id
   select id
   iddata = Get value... 1 1	; get ID of data table
   select iddata
   plus id
   Remove
endproc


procedure histogram_getcount histo_id
   select histo_id
   result = Get value... 5 1
endproc


procedure histogram_addvalue histo_id value
   select histo_id
   xstart	= Get value... 3 1
   xend		= Get value... 4 1
   nrofbins	= Get value... 2 1
   nrofcounts	= Get value... 5 1
   dx		= Get value... 6 1
   iddata	= Get value... 1 1
   Set value... 5 1 'nrofcounts'+1
   bin = floor((value-xstart)/dx) + 1
   bin = max (1, bin)
   bin = min (bin, nrofbins)
   select 'iddata'
   x = Get value... bin 1
   Set value... bin 1 'x'+1
   #printline histogram_addvalue 'value:1' to bin 'bin' ('xstart' .. 'xend')
endproc


procedure histogram_addtable histo_id table_id col
# Add values in column 'col' if table to histogram
   select table_id
   rows = Get number of rows
   for row to rows
      select table_id
      value = Get value... row col
      call histogram_addvalue histo_id value
   endfor
endproc


procedure histogram_addparam histo_id param_id
# Add values in param to histogram
   select 'histo_id'
   xstart	= Get value... 3 1
   xend		= Get value... 4 1
   nrofbins	= Get value... 2 1
   nrofcounts	= Get value... 5 1
   dx		= Get value... 6 1
   iddata	= Get value... 1 1
   select param_id
   nrframes = Get number of frames
   added = 0
   for i to nrframes
      select param_id
      value = Get value in frame... i
      if (value != undefined)
         bin = floor((value-xstart)/dx) + 1
         bin = max (1, bin)
         bin = min (bin, nrofbins)
         select iddata
         count = Get value... bin 1
         Set value... bin 1 'count'+1
         added += 1
      endif
   endfor
   select histo_id
   Set value... 5 1 'nrofcounts'+'added'
endproc


procedure histogram_calcstat histo_id
# Return mean of bins in 'result1'
   select histo_id
   iddata = Get value... 1 1
   select iddata
   result1 = Get column mean (index)... 1
endproc


procedure histogram_quantile histo_id quantile
# quantile should be in range 0..1
   select histo_id
   iddata	= Get value... 1 1
   nrofbins	= Get value... 2 1
   xstart	= Get value... 3 1
   xend		= Get value... 4 1
   dx		= Get value... 6 1
   select iddata
   total = 0
   for i to nrofbins
      value = Get value... i 1
      total += value
   endfor
   i = 0
   sum = 0
   quantile *= total
   repeat
      i += 1
      value = Get value... i 1
      sum += value
   until (sum >= quantile or i >= nrofbins)
   result = xstart + (i-1)*dx + dx/2 
endproc


procedure histogram_draw histo_id nrofmarks_left nrofmarks_bottom
   select histo_id
   iddata	= Get value... 1 1
   nrofbins	= Get value... 2 1
   xstart	= Get value... 3 1
   xend		= Get value... 4 1
   dx		= Get value... 6 1
   select iddata
   max = 0
   for i to nrofbins
      value = Get value... i 1
      max = max (value, max)
   endfor
   max = ceiling (max)
   Axes... xstart xend 0 max
   for i to nrofbins
      x2 = xstart + dx
      value = Get value... i 1
      Paint rectangle... 0.5 xstart x2 0 value
      Draw rectangle... xstart x2 0 value
      xstart += dx
   endfor
;   Marks left every... 1 1 yes yes no
;   Marks left... numberofmarks writenumbers drawticks drawdottedlines
   if (nrofmarks_left <= 0)
      nrofmarks_left = 6
   endif
   Marks left... nrofmarks_left yes yes no
;   Marks bottom every... 1 dx yes yes no 
   if (nrofmarks_bottom == 0)
      nrofmarks_bottom = ceiling(nrofbins/20)*20+1
   endif
   Marks bottom... nrofmarks_bottom yes yes no
endproc
