# stylize.praat
# Praat script for pitch contour stylization. Used by prosogram.
# Author: Piet Mertens
# Last modification: 2011-04-28


procedure create_table_of_nuclei
   n_ = nrof_nuclei_analysed
   if (n_ == 0)
      call error_msg No nuclei were found. 
      n_ = 1		; avoid crash
   endif
   nrcols = 31
   nucldatID = Create TableOfReal... nucl_data n_ nrcols
   nucleusstarttime = 1 ; starttime of nucleus
   nucleusendtime = 2	; endtime of nucleus
   nucleusdur = 3	; nucleus_duration
   f0_min = 4		; f0 min (Hz) within nucleus            before stylization
   f0_max = 5		; f0 max (Hz) within nucleus            before stylization
   f0_median = 6	; f0 median (Hz) within nucleus 	before stylization
   f0_mean = 7		; f0 mean (ST) within nucleus		before stylization
   f0_start = 8		; f0 value (Hz) at start of nucleus	after stylization
   f0_end = 9		; f0 value (Hz) at end of nucleus	after stylization
   lopitch = 10		; f0 min (Hz) within nucleus		after stylization
   hipitch = 11		; f0 max (Hz) within nucleus		after stylization
   dynamic = 12		; 0 = static, 1 = rising, -1 = falling
   intrasyllab = 13	; sum of pitch interval of tonal segments in nucleus (rises and falls compensate)
   intrasyllabup = 14	; sum of upward pitch interval of tonal segments in nucleus
   intrasyllabdown = 15	; sum of downward pitch interval of tonal segments in nucleus
   intersyllab = 16	; intersyllabic interval (between end of previous nucleus and start of current one
   trajectory = 17	; sum of absolute pitch interval of tonal segments in nucleus (rises and falls add up)
   internucleusdur = 18	; time between end of previous nucleus and start of current one
   syllabledur = 19	; syllable duration (only for appropriate segmentation method)
   gapdur = 20		; time between end of current nucleus and start of next one
   endtime_syll = 21	; used by plot_salience (only for appropriate segmentation method)
   loudness = 22	; loudness peak in nucleus (only if parameter available)
;  promL2_nucldur = 		; prominence of nucleus duration wrt left context of 2 units
;  promL2R2_nucldur = 		; prominence of nucleus duration wrt left/right context of 2 units
;  promL2_sylldur = 		; prominence of syllable duration wrt left context of 2 units
;  promL3_sylldur = 		; prominence of syllable duration wrt left context of 3 units
;  promL2R2_sylldur =		; prominence of syllable duration wrt left/right context of 2 units
;  promL2R1_sylldur =		; prominence of syllable duration wrt left/right context of 2+1 units
   promL2R1D_sylldur = 23	; prominence of syllable duration wrt dynamic left/right context of 2+1 units
;  promL3R1_sylldur =		; prominence of syllable duration wrt left/right context of 3+1 units
;  promL2R2_loudness =		; prominence of loudness wrt left/right context of 2 units
   promL2R1D_f0_mean = 24	; prominence of mean pitch (Hz) wrt dynamic left/right context of 2+1 units
   hesitation = 25		; hesitation found (only when in boundary annotation) 
   speaker_id = 26		; speaker ID number, from tier "speaker" in annotation file
   pd_nucleusdur = 27		; pairwise difference (signed) for nucleus duration
   pd_syllabledur = 28		; pairwise difference (signed) for syllable duration
   pd_voweldur = 29		; pairwise difference (signed) for vowel duration
   voweldur = 30		; vowel duration
   j_before_pause = 31		; syllable is followed by pause
   Set column label (index)... nucleusstarttime nucleusstarttime
   Set column label (index)... nucleusendtime nucleusendtime
   Set column label (index)... dynamic dynamic
   Set column label (index)... f0_start f0_start
   Set column label (index)... f0_end f0_end
   Set column label (index)... lopitch lopitch
   Set column label (index)... hipitch hipitch
   Set column label (index)... f0_min f0_min
   Set column label (index)... f0_max f0_max
   Set column label (index)... f0_median f0_median
   Set column label (index)... f0_mean f0_mean
   Set column label (index)... intrasyllab intrasyllab
   Set column label (index)... intrasyllabup intrasyllabup
   Set column label (index)... intrasyllabdown intrasyllabdown
   Set column label (index)... intersyllab intersyllab
   Set column label (index)... nucleusdur nucleusdur
   Set column label (index)... trajectory trajectory
   Set column label (index)... internucleusdur internucleusdur
;  Set column label (index)... internucleustrajectory internucleustrajectory
   Set column label (index)... syllabledur syllabledur
   Set column label (index)... gapdur gapdur
   Set column label (index)... endtime_syll endtime_syll
   Set column label (index)... loudness loudness
   Set column label (index)... promL2R1D_sylldur promL2R1D_sylldur
   Set column label (index)... promL2R1D_f0_mean promL2R1D_f0_mean
   Set column label (index)... hesitation hesitation
   Set column label (index)... speaker_id speaker_id
   Set column label (index)... pd_nucleusdur pd_nucleusdur
   Set column label (index)... pd_syllabledur pd_syllabledur
   Set column label (index)... j_before_pause before_pause
;  Set column label (index)... promL2_nucldur promL2_nucldur
;  Set column label (index)... promL2R2_nucldur promL2R2_nucldur
;  Set column label (index)... promL2_sylldur promL2_sylldur
;  Set column label (index)... promL3_sylldur promL3_sylldur
;  Set column label (index)... promL2R2_sylldur promL2R2_sylldur
;  Set column label (index)... promL2R1_sylldur promL2R1_sylldur
;  Set column label (index)... promL3R1_sylldur promL3R1_sylldur
;  Set column label (index)... promL2R2_loudness promL2R2_loudness
endproc


procedure safe_nuclei start_time_n end_time_n
# Adjust nuclei boundaries such that they are all voiced and pitch is defined
# Return in <result> the number of valid nuclei in analysis interval
   mindur_syl = 0.01	; minimum duration for syllable, otherwise skipped
   call interval_from_time nucleiID nucleus_tier start_time_n first_interval
   select nucleiID
   iNucl = first_interval
   prev_boundary = Get start point... safe_tier 1
   x1 = Get start point... nucleus_tier iNucl
   repeat
      select nucleiID
      iNucl = Get interval at time... nucleus_tier x1	; nrof intervals may change during process
      x1 = Get start point... nucleus_tier iNucl
      x2 = Get end point... nucleus_tier iNucl
      label$ = Get label of interval... nucleus_tier iNucl
      if (label$ = "a") ; "a" indicates syllabic nucleus, i.e. interval on which to apply stylization
         call defined_intersection pitchID x1 x2
         cx1 = result1
         cx2 = result2
;printline safe_nuclei: after defined_intersection x1='x1:4' x2='x2:4' cx1='cx1:4' cx2='cx2:4' result='result'
	 select nucleiID
         if (result == 0)	; nucleus fully undefined
            Set interval text... nucleus_tier iNucl undef
         else
            if (cx1 > x1)	; undefined section at start of nucleus
               Insert boundary... nucleus_tier cx1
               Set interval text... nucleus_tier iNucl xL
               Set interval text... nucleus_tier iNucl+1 a
               iNucl += 1
            endif
	    call octavejump nucleiID pitchID cx1 cx2
            cx1 = result1	; update safe interval
            cx2 = result2	; idem
;printline safe_nuclei: after octavejump cx1='cx1:4' cx2='cx2:4' result='result'
            select nucleiID
            if (result = 0)		; interval too short
               Set interval text... nucleus_tier iNucl short
            elsif (result3 == 1)	; discontinuity found
	       Insert boundary... nucleus_tier cx2
               Set interval text... nucleus_tier iNucl a
               Set interval text... nucleus_tier iNucl+1 skip
            elsif (cx2-cx1 < mindur_syl)
               Set interval text... nucleus_tier iNucl short
            elsif (cx2 < x2)
               Insert boundary... nucleus_tier cx2
               j = Get interval at time... nucleus_tier cx2
               Set interval text... nucleus_tier j xR
               Set interval text... nucleus_tier j-1 a
	    endif
	    if (cx2-cx1 >= mindur_syl)		; safe_tier
               if (cx1 > prev_boundary)
                  Insert boundary... safe_tier cx1
               endif
               Insert boundary... safe_tier cx2
               prev_boundary = cx2
               j = Get interval at time... safe_tier cx1+(cx2-cx1)/2
               Set interval text... safe_tier j a
            endif
	 endif
      endif ; label="a"
      x1 = x2
; next line required because of rounding errors !
      intervals = Get number of intervals... nucleus_tier
   until (x2 >= end_time_n or iNucl = intervals)

   select nucleiID
   nn = Count labels... nucleus_tier a
   result = nn			; number of nuclei for analysis
endproc


procedure stylize_nuclei start_time_n end_time_n
   tier = nucleus_tier
   mindur_syl = 0.01	; minimum duration for syllable, otherwise skipped
   call interval_from_time nucleiID tier start_time_n first_interval
   call interval_from_time nucleiID tier end_time_n last_interval

   select nucleiID
   nrofnuclei = Get number of intervals... tier
   if (not reuse_nucl)
      call copy_tier nucleiID nucleus_tier nucleiID pointer_tier
      call tier_clear_text nucleiID pointer_tier
   endif

   nucleus_ctr = 0
   prev_nucleus = 0	; index (into textgrid tier) of previous nucleus; 0 indicates "not found"
   for iNucl from first_interval to last_interval
      select nucleiID
      label$ = Get label of interval... tier iNucl
      x1 = Get start point... tier iNucl
      x2 = Get end point... tier iNucl
      if (label$ = "a") ; "a" indicates syllabic nucleus, i.e. interval on which to apply stylization
         call stylize_nucleus iNucl x1 x2
      endif
   endfor
   if (nrof_nuclei_analysed > 1)
      select nucldatID
      Set value... nrof_nuclei_analysed gapdur end_time_n - last_x2
   endif
endproc


procedure defined_intersection paramID t1 t2
# Find region within <t1>..<t2> for which parameter is defined
# Returns <result> = 0 when fully undefined
# Returns <result1> and <result2>, the defined interval
   result = 1
   result1 = t1
   result2 = t2
   select paramID
   dx = Get time step
   ok = 0
   t = t1
   while (ok == 0 and t <= t2)		; go right while undefined
      y = Get value at time... t Hertz Linear
      if (y == undefined)
         t += dx 
      else
         ok = 1
      endif
      result1 = t
   endwhile
   if (t > t2)
      result = 0
   else
      while (ok == 1 and t <= t2)	; go right while defined
         y = Get value at time... t Hertz Linear
         if (y == undefined)
            ok = 0
         else
            result2 = t
            t += dx 
         endif
      endwhile
      if (result1 >= t2)
         result = 0
      endif
   endif
endproc


procedure octavejump nuclID paramID t1 t2
# Find region within <t1>..<t2> for which pitch does not present discontinuities such as octave jumps
# Returns result = 0 when left with too short segment
# Returns result = 1 when OK
# Returns result1 and result2, the safe interval
# Returns result3 = 1 when discontinuity found, in which case <result2> is end of safe interval
# Stores position of discontinuity in (point) tier <discontinuity_tier> of <nuclID>
   result3 = 0			; no discontinuity found
   select paramID
   dx = Get time step
   ok = 1
   t = t1
   f1 = Get value at time... t Hertz Linear
   while (ok == 1 and t <= t2)
      select paramID
      f2 = Get value at time... t Hertz Linear
      if (abs(f2-f1)/min(f1,f2) > 0.5)
         ok = 0
         result3 = 1		; discontinuity found
	 select nuclID
         Insert point... discontinuity_tier t
      else
         result2 = t
         t += dx
         f1 = f2
      endif
   endwhile
   result = 1
   if (result2 - t1 <= dx)
      result = 0
   endif
   result1 = t1
endproc


procedure skip_cointrinsic paramID t1 t2
# Detect cointrinsic pitch at start of <t1>..<t2>
# Returns result = 1 when OK
# Returns result1, the start of safe part
   result = 1
   result1 = t1
   if (t2-t1 >= 0.04)	; need nucleus of some duration
      select paramID
      dx = Get time step
      f1 = Get value at time... t1 Hertz Linear
      t = t1 + 0.02
      repeat
         f = Get value at time... t Hertz Linear
         fr = Get value at time... max(t1,t-0.02) Hertz Linear
;df = 12 * log2 (fr/f)
;printline skip t='t' f='f:1' interv='df:3'
	 if (12 * log2 (fr/f) < 3)
            if (t-t1 < dx)
               t = t1
	    endif
	    result1 = t
         endif
         t -= dx
      until (t < t1)
   endif
endproc


procedure stylize_nucleus iNucl x1 x2
# <x1>..<x2>	times of interval to stylize

# Step 1. Find turning points (TP) in contour, by order of importance.
# A TP is the point where the distance between the actual F0 and the linear fit of F0 values is largest.
# A TP is kept only if the difference in slope between the parts before and after 
# the TP exceeds the differential glissando threshold, and if at least 
# one of the parts is an audible pitch movement.
# When a TP is found, additional TPs are searched for in the left part, until none are found.
# Then the search continues for the interval between the last TP and the end of the nucleus interval.
   nrofts = 1				; number of tonal segments
   select stylID
   Add point... x1 1
   Add point... x2 1
   i1 = Get nearest index from time... x1
   xL = x1				; xL..xR is time window under analysis
   xR = x2
;printline stylize nucleus='iNucl' start x1='x1:3' x2='x2:3' mindur_ts='mindur_ts:3'
   repeat
;printline stylize nucleus='iNucl' repeat1 xL='xL:3' xR='xR:3' mindur_ts='mindur_ts:3'
      nrofsplit = 0			; nrof turning points found in repeat loop 
      repeat				; find turning points
;printline stylize nucleus='iNucl' start xL='xL:3' xR='xR:3'
         split = 0			; nrof times split at turning point
         call is_audible "aud_A" xL xR
;printline stylize-2 xL='xL:3' xR='xR:3' AUD='aud_A' nrofts='nrofts'
	 if (aud_A)
            call turning_point xL xR
;printline stylize-3 maxdiftime='maxdiftime'
	    if (maxdiftime >= 0)	; found turning point
;d1 = maxdiftime - xL
;d2 = xR - maxdiftime
;printline stylize-4 leftpart='d1:3' rightpart='d2:3'
               if ((maxdiftime - xL >= mindur_ts) and (xR - maxdiftime >= mindur_ts))
                  call is_audible "aud_L" xL maxdiftime
	          g1 = slopeSTs		; slope (ST/s) for part left of turning point
	          call is_audible "aud_R" maxdiftime xR
                  g2 = slopeSTs		; slope (ST/s) for part right of turning point
;printline AUD_L='aud_L' slope='g1:2' AUD_R='aud_R' slope='g2:2'
                  if ((abs (g2-g1) > diffgt) and (aud_L or aud_R) )  
                     split = 1		; found a valid turning point 
                  endif
               endif
            endif
;printline stylize-5 nucleus='iNucl' split='split'
            if (split)	       
;printline stylize-5b nucleus='iNucl' split at t='maxdiftime:3'
	       select stylID
               Add point... maxdiftime 1
               xR = maxdiftime
               nrofsplit += 1		; turning points inserted in this loop
	       nrofts += 1		; additional tonal segment found
            endif
         endif
      until (split = 0)
      if (xR < x2)			; interval was split; continue segmentation for right side
         i1 += 1			; adjust xL..xR analysis window
         select stylID
	 xL = Get time from index... i1
	 xR = Get time from index... i1+1
;printline stylize-6 continue from t='xL:3'
      else				; no split...
         nrofsplit = 0			; prepare for end of repeat loop
	 xL = x2
      endif
   until (nrofsplit == 0 and xL >= x2)

# Step 2. Actual stylisation
   xmid = x1 + (x2-x1)/2
   select nucleiID
   cum_intra = 0		; cumulated intrasyllabic pitch variation
   cum_intra_up = 0		; sum of intrasyllabic pitch rises
   cum_intra_down = 0		; sum of intrasyllabic pitch falls
   cum_abs_intra = 0		; cumulated absolute intrasyllabic pitch variation
;   cum_abs_inter = 0		; cumulated absolute intersyllabic pitch variation
   dynamic_type = 0		; dynamic_type, 0 = static, 1 = rising, -1 = falling
   dynamic_up = 0
   dynamic_down = 0
   select stylID
   i = Get nearest index from time... x1
   i2 = Get nearest index from time... x2
   ts = 1			; index of tonal segment under analysis
   while (i < i2)		; for each tonal segment
      select stylID
      xL = Get time from index... i
      xR = Get time from index... i+1
      call is_audible "aud_A" xL xR
      intST = dist		; pitch interval (in ST) in current tonal segment
      cum_intra += intST
      cum_abs_intra += abs (intST)
      cum_intra_up += max (intST, 0)
      cum_intra_down += min (intST, 0)
      # Check special case of two inaudible parts. e.g. bell-shaped contour
      if (aud_A = 1 and nrofts = 1)
;printline Check special case xL='xL:3' xR='xR:3'
            call turning_point xL xR
;printline special_case maxdiftime='maxdiftime:3'
	    if (maxdiftime >= 0)	; turning point found
               call is_audible "aud_L" xL maxdiftime
	       g1 = slopeSTs
               call is_audible "aud_R" maxdiftime xR
               g2 = slopeSTs
               d1 = maxdiftime - xL
               d2 = xR - maxdiftime
;printline AUD_L='aud_L' slope='g1:2' AUD_R='aud_R' slope='g2:2'
               if (aud_L == 0 and aud_R == 0)  
                  if ((g1 > 0 and g2 < 0) or (g1 < 0 and g2 > 0))  
                     aud_A = 0	; consider inaudible
                     intST = 0
	          endif
	       endif
            endif
      endif ; special case
      select pitchID
      yR = Get value at time... xR Hertz Linear
      yL = Get value at time... xL Hertz Linear
      yM = Get quantile... xL xR 0.5 Hertz
;printline xL='xL:3' yL='yL:3' xR='xR:3' yR='yR:3' yM='yM:3'
      select stylID
      if (ts = 1)		; first tonal segment of nucleus => also set value at start
	 if (aud_A = 0)
            yR = yM		; normalize pitch to modal pitch
            yL = yR
         endif
         Remove point... i	; to replace value of point at xL
	 Add point... xL yL	; set Y value of turning point at xL
         pv_lo = min (yL, yR)
         pv_hi = max (yL, yR)
         pv_start = yL
      endif
      Remove point... i+1	; to replace value of point at xR
      Add point... xR yR	; set Y value of turning point at xR
      if (aud_A)
         distST = 12 * log2 (yR/yL)
         if (distST > 0)
            dynamic_up += intST
	 else
            dynamic_down += intST
         endif
         dynamic_type = 1
      endif
      pv_lo = min (pv_lo, yR)
      pv_hi = max (pv_hi, yR)
      if (contour_annotation)
         select nucleiID
         imid = Get interval at time... contour_tier xmid
         call contour_analysis ts aud_A intST duration nucleiID contour_tier imid
      endif
      ts += 1
      i += 1
   endwhile
   if (dynamic_type == 1)
      if (abs(dynamic_down) > dynamic_up)
         dynamic_type = -1
      endif
   endif
   select pitchID
   v_f0_min = Get minimum... x1 x2 Hertz Parabolic
   v_f0_max = Get maximum... x1 x2 Hertz Parabolic
   pv_median = Get quantile... x1 x2 0.50 Hertz
   pv_mean = Get mean... x1 x2 semitones re 1 Hz
   if (prev_nucleus > 0)	; already found a nucleus in left context
      call distST prev_f0_end pv_start
      select nucleiID
      prev_x2 = Get end point... nucleus_tier prev_nucleus
   else				; first nucleus in analysis window
      prev_x2 = anal_t1
      distST = 0
   endif
   if (syllables_available)
      select nucleiID
      imid = Get interval at time... syllable_tier xmid
      syllt2 = Get end point... syllable_tier imid
      syllt1 = Get start point... syllable_tier imid
      sylldur = syllt2 - syllt1
   else
      sylldur = undefined
      syllt2 = undefined
   endif
   nucleus_ctr += 1
   select nucleiID
   Set interval text... pointer_tier iNucl 'nucleus_ctr'
   select nucldatID
   Set row label (index)... nucleus_ctr 'x1:3'
   s$ = fixed$(x1,3)
   Set value... nucleus_ctr nucleusstarttime 's$'
   s$ = fixed$(x2,3)
   Set value... nucleus_ctr nucleusendtime 's$'
   ; s$ = fixed$(x2-x1,3)
   ; Set value... nucleus_ctr nucleusdur 's$'
   Set value... nucleus_ctr nucleusdur x2-x1
   Set value... nucleus_ctr f0_start floor(pv_start)
   Set value... nucleus_ctr f0_end floor(yR)
   Set value... nucleus_ctr f0_min floor(v_f0_min)
   Set value... nucleus_ctr f0_max floor(v_f0_max)
   Set value... nucleus_ctr f0_median floor(pv_median)
   s$ = fixed$(pv_mean,3)
   Set value... nucleus_ctr f0_mean 's$'
   Set value... nucleus_ctr lopitch floor(pv_lo)
   Set value... nucleus_ctr hipitch floor(pv_hi)
   Set value... nucleus_ctr dynamic dynamic_type
   s$ = fixed$(cum_intra,3)
   Set value... nucleus_ctr intrasyllab 's$'
   s$ = fixed$(cum_intra_up,3)
   Set value... nucleus_ctr intrasyllabup 's$'
   s$ = fixed$(cum_intra_down,3)
   Set value... nucleus_ctr intrasyllabdown 's$'
   s$ = fixed$(distST,3)
   Set value... nucleus_ctr intersyllab 's$'
   s$ = fixed$(cum_abs_intra,3)
   Set value... nucleus_ctr trajectory 's$'
;   internucleusdur = 18	; time between end of previous nucleus and start of current one
   Set value... nucleus_ctr internucleusdur x1 - prev_x2
   Set value... nucleus_ctr syllabledur sylldur
;    gapdur = 20		; time between end of current nucleus and start of next one
;   endtime_syll = 21	; used by plot_salience (only for appropriate segmentation method)
   Set value... nucleus_ctr endtime_syll syllt2
;    loudness = 22	; loudness peak in nucleus (only if parameter available)
   if (include_loudness and loudness_available)
      select loudnessID
      v = Get maximum... x1 x2 None
      s$ = fixed$(v,3)
      select nucldatID
      Set value... nucleus_ctr loudness 's$'
   endif
   if (prev_nucleus > 0)	; already found a nucleus in left context
      Set value... prev_nucleus_ctr gapdur x1 - prev_x2
   endif
   prev_f0_end = yR			; save f0_end for next syllable
   prev_nucleus = iNucl			; used for next nucleus
   prev_nucleus_ctr = nucleus_ctr	; used for next nucleus
   last_x2 = x2				; used for last nucleus
   if (use_profile_histo)
      dur_ = x2-x1
      call profile_hist_addvalue cum_intra cum_abs_intra distST dur_
   endif
endproc


procedure distST f1 f2
   distST = 12 * log2 (f2/f1)
endproc


procedure slopeSTs first last
# Calculate slope of F0 variation (in ST/s) in time interval <first>..<last>
# Return slope in variable 'slopeSTs'.
# Return pitch intervak (in ST) in variable 'dist'
   select pitchID
   max = Get maximum... first last Hertz None
   min = Get minimum... first last Hertz None
   tmax = Get time of maximum... first last Hertz None
   tmin = Get time of minimum... first last Hertz None
   if (tmin <= tmax) 
      dist = 12 * log2 (max/min)
   else 
      dist = 12 * log2 (min/max)
   endif
   slopeSTs = dist / (last-first)
endproc


procedure is_audible varname$ first last
# Evaluate audibility of pitch change for frequency values (in Hz)
# and store boolean result in variable with name 'varname$'
   duration = last-first
   call slopeSTs first last
   if (abs (slopeSTs) >= glissando/(duration*duration))
      'varname$' = 1
   else 
      'varname$' = 0
   endif
endproc


procedure turning_point first last
# Find most important turning point.
# returns <maxdiftime> = time of turning point in time interval <first>..<last>
# returns -1 if max difference too small ( < 1 ST ) 
   select pitchID
   dx = Get time step
   q1 = Get frame number from time... first
   f0_x1 = Get value in frame... q1 Hertz
   if (f0_x1 == undefined)
      x1_time = Get time from frame... q1
      call fatal_error turning_point: Pitch undefined at (left boundary) time='x1_time' first='first' last='last' q1='q1' q2='q2' 
   endif
   q2 = Get frame number from time... last
   f0_x2 = Get value in frame... q2 Hertz
   if (f0_x2 == undefined)
      x2_time = Get time from frame... q2
      call fatal_error turning_point: Pitch undefined at (right boundary) time='x2_time' first='first' last='last' q1='q1' q2='q2' 
   endif
   a = f0_x1
   b = (f0_x2 - f0_x1) / ((q2-q1)*dx)
   maxdif = 0
   maxdiffit = 1
   maxdiftime = first
   maxdifq = q1
   for q from q1 to q2
      fit = a + b * ((q-q1)*dx)
      f0_x = Get value in frame... q Hertz
      if (f0_x == undefined)
         time = Get time from frame... q
	 call fatal_error turning_point: Pitch undefined at time='time:4' first='first' last='last' q1='q1' q2='q2' 
      endif
      dy = abs (f0_x - fit)
      if (dy > maxdif)
         maxdif = dy
	 maxdifq = q
	 maxdiffit = fit
      endif
   endfor
   if (maxdif = 0)
      maxdiftime = -1
   elsif (abs(12 * log2 (maxdif/maxdiffit)) < 1)	; smaller than 1 ST 
      maxdiftime = -1
   else
      maxdiftime = Get time from frame... maxdifq
   endif
endproc


procedure calc_vowel_duration table dst
# dst		column where results are stored
   select table
   rows = Get number of rows
   for row to rows	; for each nuclei in the signal
      s_$ = ""
      select table
      x1 = Get value... row nucleusstarttime
      x2 = Get value... row nucleusendtime
      xmid = x1 + (x2-x1)/2
      select nucleiID
      i_ = Get interval at time... syllable_tier xmid
      s_$ = Get label of interval... syllable_tier i_ 
      x1 = Get start point... syllable_tier i_
      x2 = Get end point... syllable_tier i_
      i = Get interval at time... phone_tier x1
      i2 = Get interval at time... phone_tier x2
      repeat
         label$ = Get label of interval... phone_tier i 
         call is_vowel 'label$'
	 if (is_vowel)
            x1 = Get start point... phone_tier i
            x2 = Get end point... phone_tier i
            select table
            Set value... row dst x2-x1
         elsif (i+1 == i2)
            call msg No vowel in syllable <'s_$'> at 'xmid:3'
	 endif
	 i += 1
      until (is_vowel or i == i2)
   endfor
endproc


procedure calc_nPVI table src dst
# Normalized Pairwise Variability Index
# src		column where values are taken from
# dst		column where results are stored
   select table
   rows = Get number of rows
   sum = 0
   for row to rows	; for each nuclei in the signal
      y = Get value... row src
      diff = 0 ; default when no right context
      if (row < rows)
         y2 = Get value... row+1 src 
         v = (y - y2) / ((y + y2)/2)
      endif
      Set value... row dst v
      sum += abs(v)
   endfor
   result = sum/(rows-1)
endproc


procedure prosodic_profile
 if (nrof_nuclei_analysed > 0)
   min_pause_duration = 0.3
   select nucldatID
   nrof_nucl = Get number of rows
   sum_intra = 0	; intrasyllabic interval
   sum_mov_inter = 0	; intersyllabic interval
   sum_traj_intra = 0	; intrasyllabic trajectory
   sum_traj_inter = 0	; intersyllabic trajectory
   sum_traj_all = 0	; trajectory of both intersyllabic and intrasyllabic intervals, including pauses
   sum_traj_phon = 0	; trajectory of both intersyllabic and intrasyllabic intervals, excluding pauses
   sum_nucldur = 0
   sum_internucldur = 0	; time between nuclei, corrected for pauses
   sum_gapdur = 0	; time of pauses
   nrof_dyn = 0
   nrof_up = 0
   nrof_down = 0
   maxp = 0
   minp = 1000
   sum_median_p = 0
;printline Profile nrof_nucl='nrof_nucl'


   select nucldatID
   for j from 1 to nrof_nucl
      x1$ = Get row label... j
      sum_nucldur += Get value... j nucleusdur
      sum_intra += Get value... j intrasyllab
      v = Get value... j lopitch
      minp = min (minp, v) 
      v = Get value... j hipitch
      maxp = max (maxp, v) 
      sum_median_p += Get value... j f0_median
      v_dyn = Get value... j dynamic
      if (v_dyn > 0)
         nrof_dyn += 1
         nrof_up += 1
      elsif (v_dyn < 0)
         nrof_dyn += 1
         nrof_down += 1
      endif
      v_traj = Get value... j trajectory
      v_mov_inter = Get value... j intersyllab
      v_ind = Get value... j internucleusdur
      if (v_ind >= min_pause_duration)	; pause to LEFT of current nucleus
	 sum_gapdur += v_ind
      else
         sum_internucldur += v_ind
         sum_traj_phon += abs(v_mov_inter) + v_traj
      endif
      sum_traj_inter += abs(v_mov_inter)
      sum_mov_inter += v_mov_inter
      sum_traj_intra += v_traj
      sum_traj_all += abs(v_mov_inter) + v_traj
   endfor
   total_dur      = anal_t2 - anal_t1
   phonation_time = sum_internucldur + sum_nucldur
   speech_time    = phonation_time + sum_gapdur
   prop_pause     = 100 * sum_gapdur / speech_time
   prop_phonation = 100 * phonation_time / speech_time
   mean_of_medianp = sum_median_p / nrof_nucl

# Compute quantiles of median pitch values of each nucleus before stylization
   select nucldatID
   Extract column ranges... 'f0_median'
   tmpID1 = selected ("TableOfReal",-1)
   To Table... 1
   # We now have 2 columns because row labels of TableOfReal are used as column 1
   tmpID2 = selected ("Table",-1)
   Set column label (index)... 2 pitch
   mpqu50 = Get quantile... pitch 0.5
   mpqu05 = Get quantile... pitch 0.05
   mpqu95 = Get quantile... pitch 0.95
   mpqu02 = Get quantile... pitch 0.02
   mpqu98 = Get quantile... pitch 0.98
   select tmpID1
   plus tmpID2
   Remove
   ;printline "f0_median quantiles: (.02)='mpqu02:3' (.05)='mpqu05:3' (.50)='mpqu50:3' (.95)='mpqu95:3' (.98)='mpqu98:3'
   select nucldatID

# Compute quantiles of f0_min and f0_max values of each nucleus before stylization
   tmptableID = Create Table without column names... pitchvalues nrof_nucl*2 1
   Set column label (index)... 1 pitchvalue
   for j from 1 to nrof_nucl
      select nucldatID
      vlo = Get value... j f0_min
      vhi = Get value... j f0_max
      select tmptableID
      Set numeric value... j pitchvalue vlo
      Set numeric value... (nrof_nucl+j) pitchvalue vhi
   endfor
   rpqu50 = Get quantile... pitchvalue 0.5
   rpqu05 = Get quantile... pitchvalue 0.05
   rpqu95 = Get quantile... pitchvalue 0.95
   rpqu02 = Get quantile... pitchvalue 0.02
   rpqu98 = Get quantile... pitchvalue 0.98
   select tmptableID
   Remove

# Compute quantiles of lo and hi pitch values of each nucleus after stylization
   tmptableID = Create Table without column names... pitchvalues nrof_nucl*2 1
   Set column label (index)... 1 pitchvalue
   for j from 1 to nrof_nucl
      select nucldatID
      vlo = Get value... j lopitch
      vhi = Get value... j hipitch
      select tmptableID
      Set numeric value... j pitchvalue vlo
      Set numeric value... (nrof_nucl+j) pitchvalue vhi
   endfor
   spqu50 = Get quantile... pitchvalue 0.5
   spqu05 = Get quantile... pitchvalue 0.05
   spqu95 = Get quantile... pitchvalue 0.95
   spqu02 = Get quantile... pitchvalue 0.02
   spqu98 = Get quantile... pitchvalue 0.98
   select tmptableID
   Remove


   date_$ = date$ ()
printline Writing prosodic profile to: 'statsfile$'
   filedelete 'statsfile$'
   w$ = "fileappend ""'statsfile$'"" 'newline$'"
   'w$' Prosodic profile for input file: 'signalfile$'
   'w$'
   'w$' Date: 'date_$'
   'w$' Segmentation type: 'segmentation_name$'
   'w$' Time:  
   'w$'     total speech time='speech_time:2'(s) [Note 1]
   'w$'     estimated phonation time='phonation_time:2' ('prop_phonation:2'% of speech time) [Note 2]
   'w$'     estimated pause time='sum_gapdur:2' ('prop_pause:2'% of speech time) [Note 3]

   'w$' Nucleus: 'nrof_nucl' nuclei in signal
   if (nrof_nucl < 100)
      'w$' 'newline$' WARNING: The global measures below are only meaningful for speech samples of at least 100 nuclei (syllables).'newline$'
   endif
   select nucldatID
   mean_nucldur = Get column mean (label)... nucleusdur
   stdev_nucldur = Get column stdev (label)... nucleusdur
   'w$' Duration: 
   'w$'     Nucleus duration: 
   'w$'        mean='mean_nucldur:3'(s) stdev='stdev_nucldur:3' summed nucleus duration='sum_nucldur:2'(s)
   call calc_nPVI nucldatID nucleusdur pd_nucleusdur
   'w$'     nPVI (nucleus duration)='result:3' 
   if (syllables_available)
      call calc_nPVI nucldatID syllabledur pd_syllabledur
     'w$'     nPVI (syllable duration)='result:3' 
   endif
   if (syllables_available and phones_available)
      call calc_vowel_duration nucldatID voweldur
      call calc_nPVI nucldatID voweldur pd_voweldur
     'w$'     nPVI (vowel duration)='result:3' 
   endif

   'w$' Global pitch measures: 
   'w$'     Quantiles of min and max F0 values of nuclei before stylisation: 
   'w$'       2%='rpqu02:2'(Hz), 5%='rpqu05:2'(Hz), 50%='rpqu50:2'(Hz), 95%='rpqu95:2'(Hz), 98%='rpqu98:2'(Hz) 
   'w$'     Quantiles for median pitch values of nuclei before stylisation: 
   'w$'       2%='mpqu02:2'(Hz), 5%='mpqu05:2'(Hz), 50%='mpqu50:2'(Hz), 95%='mpqu95:2'(Hz), 98%='mpqu98:2'(Hz) 
   'w$'     Pitch measures for stylized nuclei: 
   'w$'       min='minp:2'(Hz), max='maxp:2'(Hz), mean of median='mean_of_medianp:2' [Note 7]
   'w$'     Quantiles of low & high pitch values of nuclei after stylisation: 
   'w$'       2%='spqu02:2'(Hz), 5%='spqu05:2'(Hz), 50%='spqu50:2'(Hz), 95%='spqu95:2'(Hz), 98%='spqu98:2'(Hz) 

   'w$' Intrasyllabic pitch interval: 
   prop_dyn = 100 * nrof_dyn/nrof_nucl
   if (nrof_dyn > 0)
      prop_up = 100*nrof_up/nrof_dyn
      prop_down = 100*nrof_down/nrof_dyn
   else
      prop_up = 0
      prop_down = 0
   endif
   'w$'     dynamic='prop_dyn:2'%, of which: rising='prop_up:2'%, falling='prop_down:2'% [Note 4]
   select nucldatID
   mean = Get column mean (label)... intrasyllab
   stdev = Get column stdev (label)... intrasyllab
   'w$'     mean='mean:2'(ST) stdev='stdev:2'
   rate = sum_intra/sum_nucldur
   'w$'     sum='sum_intra:2'(ST) sum/nucleus_time='rate:2'(ST/s) [Note 5]
   traj_intra_rate = sum_traj_intra/sum_nucldur
   'w$'     trajectory='sum_traj_intra:2'(ST) trajectory/nucleus_time='traj_intra_rate:2'(ST/s) [Note 6]

   'w$' Intersyllabic pitch interval: 
   mean = Get column mean (label)... intersyllab
   stdev = Get column stdev (label)... intersyllab
   if (sum_internucldur > 0)
      rate = sum_mov_inter/sum_internucldur
      traj_inter_rate = sum_traj_inter/sum_internucldur
   else
      rate = 0
      traj_inter_rate = 0
   endif
;printline mov_inter_rate='rate' sum='sum_mov_inter' dur='sum_internucldur'
   'w$'     mean='mean:2'(ST) stdev='stdev:2'
   'w$'     sum='sum_mov_inter:2'(ST) sum/inter_nucleus_time='rate:2'(ST/s) [Note 5]
   'w$'     trajectory='sum_traj_inter:2'(ST) trajectory/inter_nucleus_time='traj_inter_rate:2'(ST/s) [Note 6]

   'w$' All pitch intervals: 
   traj_all_rate = sum_traj_all/speech_time
   traj_phon_rate = sum_traj_phon/phonation_time
   'w$'     trajectory(speech)='sum_traj_all:2'(ST) [Note 1,2,6]
   'w$'     trajectory(speech)/speech_time='traj_all_rate:2'(ST/s)
   'w$'     trajectory(phonation)/phonation_time='traj_phon_rate:2'(ST/s)
   'w$'
   'w$' Notes 
   'w$'  [1] speech time = internucleus time + intranucleus time + pause time 
   'w$'  [2] phonation time = internucleus time + intranucleus time 
   'w$'  [3] estimated pause = when internucleus time >= 'min_pause_duration'
   'w$'  [4] dynamic = when pitch variation exceeds glissando threshold
   'w$'      rising = when sum of upward intervals > sum of abs(downward intervals)
   'w$'  [5] sum = sum of signed pitch intervals (falls and rises compensate)
   'w$'  [6] trajectory = sum of absolute pitch intervals (falls and rises add up)
   'w$'  [7] mean of median = mean of median pitch of all nuclei
   'w$'

# store some values for the global profile spreadsheet (covering multiple speech files) 
   select reportID
   Set row label (index)... iFile "'basename$'"
   Set value... iFile speechtime 'speech_time:3'
   Set value... iFile propphonation 'prop_phonation:2'
   Set value... iFile proppause 'prop_pause:2'
   Set value... iFile meannucldur 'mean_nucldur:3'
   Set value... iFile hipitchglobal 'maxp:2'
   Set value... iFile lowpitchglobal 'minp:2'
   Set value... iFile meanpitchglobal 'mean_of_medianp:2'
   Set value... iFile f0_qu02 'rpqu02:2'
   Set value... iFile f0_qu05 'rpqu05:2'
   Set value... iFile f0_qu50 'rpqu50:2'
   Set value... iFile f0_qu95 'rpqu95:2'
   Set value... iFile f0_qu98 'rpqu98:2'
   Set value... iFile f0_medianqu02 'mpqu02:2'
   Set value... iFile f0_medianqu05 'mpqu05:2'
   Set value... iFile f0_medianqu50 'mpqu50:2'
   Set value... iFile f0_medianqu95 'mpqu95:2'
   Set value... iFile f0_medianqu98 'mpqu98:2'
   Set value... iFile sp_qu02 'spqu02:2'
   Set value... iFile sp_qu05 'spqu05:2'
   Set value... iFile sp_qu50 'spqu50:2'
   Set value... iFile sp_qu95 'spqu95:2'
   Set value... iFile sp_qu98 'spqu98:2'
   Set value... iFile propdynamic 'prop_dyn:2'
   Set value... iFile propup 'prop_up:2'
   Set value... iFile propdown 'prop_down:2'
   Set value... iFile ratetrajintra 'traj_intra_rate:2'
   Set value... iFile ratetrajinter 'traj_inter_rate:2'
   Set value... iFile ratetrajall 'traj_all_rate:2'
   Set value... iFile ratetrajphon 'traj_phon_rate:2'

   if (use_profile_histo)
;      call profile_hist_draw
   endif
 else ; (nrof_nuclei_analysed = 0)
   filedelete 'statsfile$'
   fileappend 'statsfile$' Prosodic profile for input file: 'signalfile$'
   fileappend 'statsfile$' The prosodic profile could not be calculated because no nuclei were detected.'newline$'
 endif
endproc


procedure profile_hist_init
   ; histogram nrofbins xstart xend
   call histogram_create 20 -10 10
   id_hist_cintra = result
   call histogram_create 20 0 10
   id_hist_cabsintra = result
   call histogram_create 20 -10 10
   id_hist_inter = result
   call histogram_create 20 0.05 0.25
   id_hist_nucldur = result
endproc

procedure profile_hist_addvalue value1 value2 value3 value4
   ; histogram_addvalue histo_id value
   call histogram_addvalue id_hist_cintra value1
   call histogram_addvalue id_hist_cabsintra value2
   call histogram_addvalue id_hist_inter value3
   call histogram_addvalue id_hist_nucldur value4
endproc

procedure profile_hist_remove
   call histogram_remove id_hist_cintra
   call histogram_remove id_hist_cabsintra
   call histogram_remove id_hist_inter
   call histogram_remove id_hist_nucldur
endproc

procedure profile_hist_draw
   Erase all
   Select outer viewport... 0 5 0 2.5
   Viewport text... Centre Top 0 'fname$'
   call histogram_draw id_hist_cintra 6 0
   Viewport text... Centre Bottom 0 Cumulated intrasyllabic pitch change
   Select outer viewport... 0 5 2.5 5
   call histogram_draw id_hist_cabsintra 6 0
   Viewport text... Centre Bottom 0 Cumulated intrasyllabic absolute pitch change
   Select outer viewport... 0 5 5 7.5
   call histogram_draw id_hist_inter 6 0
   Viewport text... Centre Bottom 0 Intersyllabic pitch change
   Select outer viewport... 0 5 7.5 10
   call histogram_draw id_hist_nucldur 6 0
   Viewport text... Centre Bottom 0 Nucleus duration
   Select outer viewport... 0 5 0 10
endproc


procedure pitchrange_speakers
; Compute pitch range for each speaker using high and low pitch values of each syllable.
; Should be called after stylization.
   if (speakers < 1)
      call error_msg Expected >= 1 speaker
   endif
   call msg Pitch range of speakers...

   for speaker_j from 1 to speakers

; For speaker X, compute quantiles of low and high pitch values of each nucleus after stylization, 
; using a temporary table.
; Avoid nuclei with a pitch discontinuity.
      select nucleiID
      k = Get number of points... discontinuity_tier
      select nucldatID
      rows = Get number of rows
      rows2 = rows * 2		; use 2 pitch values per syllable
      tmptableID = Create Table without column names... pitchvalues rows2 1
      Set column label (index)... 1 pitchvalue
      n = 0
      for j from 1 to rows
         select nucldatID
         id = Get value... j speaker_id
	 if (id == speaker_j)
           t1_ = Get value... j nucleusstarttime
           t2_ = Get value... j nucleusendtime
	   vlo = Get value... j lopitch
           vhi = Get value... j hipitch
           select nucleiID
	   i_ = Get high index from time... discontinuity_tier t1_
           if (i_ > 0 and i_ <= k)		; time t1_ > time of last discontinuity
	      t_ = Get time of point... discontinuity_tier i_
           else
              t_ = -1 
	   endif
;printline pitchrange: t1='t1_:2' t2='t2_:2' i='i_' t='t_:2'
           if (t_ >= t1_-time_step and t_ <= t2_+time_step)	; avoid data at discontinuity
;printline pitchrange: skipped nucleus at 't1_:2'
	   else
	      select tmptableID
              Set numeric value... n+1 pitchvalue vlo
              Set numeric value... n+2 pitchvalue vhi
              n += 2
           endif
	 endif
      endfor
      select tmptableID
      if (n < rows2)		; number of row in table should match number of values
         row = rows2
	 while (row >= n+1)
            Remove row... row
            row -= 1
	 endwhile
      endif
      q50 = Get quantile... pitchvalue 0.5
      q02 = Get quantile... pitchvalue 0.02
      q98 = Get quantile... pitchvalue 0.98
      mean = Get mean... pitchvalue
      stdev = Get standard deviation... pitchvalue
      q50ST = hertzToSemitones (q50) - hertzToSemitones(1)
      q02ST = hertzToSemitones (q02) - hertzToSemitones(1)
      q98ST = hertzToSemitones (q98) - hertzToSemitones(1)
      meanST = hertzToSemitones (mean) - hertzToSemitones(1)
      select tmptableID
      Remove
      n_ = n/2			; nrofnuclei used 
      label$ = speaker_label'speaker_j'$
      range = q98ST - q02ST
      speaker_range_'speaker_j'$ = "BOTTOM_Hz='q02:0' BOTTOM_ST='q02ST:1' MEDIAN_Hz='q50:0' MEDIAN_ST='q50ST:1' MEAN_Hz='mean:0' MEAN_ST='meanST:1' TOP_Hz='q98:0' TOP_ST='q98ST:1' RANGE_ST='range:1' NROFNUCL='n_'"
      printline #'speaker_j' "'label$'" BOTTOM_ST='q02ST:1' MEDIAN_ST='q50ST:1' MEAN_ST='meanST:1' TOP_ST='q98ST:1' RANGE='range:1' NROFNUCL='n_'
printline #'speaker_j' "'label$'" std dev (Hz) = 'stdev:0'

; For speaker X, compute mean and stdev for intrasyllabic and intersyllabic pitch variation, using a temporary copy of nucldat
      select nucldatID
      tmptableID = Copy... tmptable
      rows = Get number of rows
      for j from 1 to rows
	 spkr = Get value... j speaker_id
         if (j > 1)
            prev = Get value... j-1 speaker_id
	    if (spkr == speaker_j and spkr <> prev)	; speaker has changed
               Set value... j intersyllab 0
            endif
         endif
	 v = Get value... j intersyllab
         Set value... j intersyllab abs(v)
      endfor
      j = 1
      while (j <= rows)		; discard info from other speakers
	 spkr = Get value... j speaker_id
	 if (spkr <> speaker_j)
            Remove row (index)... j
            rows -= 1
            j -= 1
         endif
         j += 1
      endwhile
      mean_intra_up = Get column mean (index)... intrasyllabup 
      stdev_intra_up = Get column stdev (index)... intrasyllabup 
      mean_abs_inter = Get column mean (index)... intersyllab	; meanwhile tweeked
      stdev_abs_inter = Get column stdev (index)... intersyllab 
      select tmptableID
      Remove
      label$ = speaker_label'speaker_j'$
      speaker_profile_'speaker_j'$ = "M_INTRA_UP='mean_intra_up:1' S_INTRA_UP='stdev_intra_up:2' M_ABS_INTER='mean_abs_inter:1' S_ABS_INTER='stdev_abs_inter:2'"
      printline #'speaker_j' "'label$'" M_INTRA_UP='mean_intra_up:1' S_INTRA_UP='stdev_intra_up:2' M_ABS_INTER='mean_abs_inter:1' S_ABS_INTER='stdev_abs_inter:2'

   endfor ; for speaker_j
endproc
